import java.io.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.filechooser.*;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.StringTokenizer;

import java.beans.PropertyVetoException;

import ExStockData.Graph.*;
import ExStockData.XML.XMLStream;
import ExStockData.XML.StreamDialog;

public class ExStockDataInterface extends JFrame implements ActionListener, ItemListener {
    public static int XBOUND,YBOUND;
    public static int openFrameCount = 0;
    
    JMenuBar menuBar;
    JDesktopPane desktop;
    JFileChooser fileChooser;
    JMenu realTimeMenu;
    JMenuItem dayStockItem, analyzeStockItem, analyzeSortedStockItem, 
	weatherItem, nonUniqueSDItem, combineNonUniqueSDItem;	
    
    public ExStockDataInterface() {
        super("Stock Data Extractor");
	
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(inset, inset,
                  screenSize.width  - inset*2,
                  screenSize.height - inset*2);
	
	XBOUND = (int)(screenSize.width*0.8);
	YBOUND = (int)(screenSize.height*0.8);

        desktop = new JDesktopPane();
        setContentPane(desktop);
        setJMenuBar(createMenuBar());
	
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
    }    
    
    public JMenuBar createMenuBar() {    	
	menuBar = new JMenuBar();
        fileChooser = new JFileChooser();
	
	realTimeMenu = new JMenu("Stock");
	realTimeMenu.getAccessibleContext().setAccessibleDescription("Select Action");
	
	stockItem = new JMenuItem("Current stock data...");
	stockItem.addActionListener(this);
	realTimeMenu.add(stockItem);
	
	dayStockItem = new JMenuItem("One day stock data...");
	dayStockItem.addActionListener(this);
	realTimeMenu.add(dayStockItem);
	
	analyzeStockItem = new JMenuItem("Analyze a .sto file...");
	analyzeStockItem.addActionListener(this);
	realTimeMenu.add(analyzeStockItem);
	
	analyzeSortedStockItem = new JMenuItem("Analyze a SortedAnalyze.txt file...");
	analyzeSortedStockItem.addActionListener(this);
	realTimeMenu.add(analyzeSortedStockItem);
	
	menuBar.add(realTimeMenu);
	return menuBar;
    }
    
    public void actionPerformed(ActionEvent e) {
    	if (e.getSource() == dayStockItem) {
	    
	    StreamDialog dialog = new StreamDialog(this);
	    if(dialog.getDayStock())
		JOptionPane.showMessageDialog(this,
		    "Stock data file (.sto) is stored in stockdata directory.",
		    "Success Stock Data Retrieval", JOptionPane.PLAIN_MESSAGE);
	    
	} else if(e.getSource() == analyzeStockItem) {
		    
	    int returnVal = fileChooser.showOpenDialog(this);
	    
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fileChooser.getSelectedFile();
		
		String[][] str = fileToMatrix(file, 3);
		
		long[] sales = new long[str.length];
		for(int i = 0; i<str.length; i++) {
		    sales[i] = Long.parseLong(str[i][1]);
		}
		
		Graph g = new Graph("Stock Trading Popularity (by Total Share Volume)");
		g.setVisible(true);
		g.plot(sales);
		desktop.add(g);
		try {
		    g.setSelected(true);
		} catch (PropertyVetoException exc) {}
		
		File[] stockFiles = new File[str.length];
		for(int i = 0; i<str.length; i++) {
		    stockFiles[i] = new File("stockdata//" + str[i][2] + ".txt");
		}
		
		str = null;
		sales = null;
		g = null;
		System.gc();
		
		StockTuple[] stockTuple = multipleFilesToMatrix(stockFiles, 6);
		
		Arrays.sort(stockTuple, new StockTupleComparator());
		
		saveArrayToFile(stockTuple);
		
		String[] sstr = new String[stockTuple.length];
		for(int i = 0; i<stockTuple.length; i++) {
		    sstr[i] = String.valueOf(stockTuple[i].name);
		}
		
		stockTuple = null;
		System.gc();
		
		Popularity pop = new Popularity();
		long[] p = pop.calcPopularity(sstr);
		saveArrayToFile(p, "stockdata//savepopularity");
		long reverse;
		for(int i = 0; i<p.length/2; i++) {
		    reverse = p[i];
		    p[i] = p[p.length-1-i];
		    p[p.length-1-i] = reverse;
		}
		g = new Graph("Stock Data Popularity (by Number of Transaction)");
		g.setVisible(true);
		g.plot(p);
		desktop.add(g);
		try {
		    g.setSelected(true);
		} catch (PropertyVetoException exc) {}	    	    
		
		StackDistance sdobj = new StackDistance();
		long[] sd = sdobj.calcStackDistance(sstr);
		saveArrayToFile(sd, "stockdata//savestackdistance");
		g = new Graph("Stock Data Stack Distance");
		g.setVisible(true);
		g.plot(sd);
		desktop.add(g);
		try {
		    g.setSelected(true);
		} catch (PropertyVetoException exc) {}
		
	    }
	    
	} else if (e.getSource() == analyzeSortedStockItem) {
	    
	    int returnVal = fileChooser.showOpenDialog(this);
	    
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fileChooser.getSelectedFile();
		
		BufferedReader in = null;
		String s = null;
		ArrayList arr = new ArrayList();
		try {
		    in = new BufferedReader(new FileReader(file));
		    s = in.readLine();
		    while(s != null) {
			arr.add(s.substring(0,s.indexOf(" ")));
			s = in.readLine();
			    }
		} catch (Exception ee) {
		    ee.printStackTrace();
		}
		
		String[] sstr = new String[arr.size()];
		for(int i = 0; i<arr.size(); i++) {
		    sstr[i] = (String)arr.get(i);
		}
		
		arr = null;
		System.gc();
		
		Popularity pop = new Popularity();
		long[] p = pop.calcPopularity(sstr);
		saveArrayToFile(p, "stockdata//savepopularity");
		long reverse;
		for(int i = 0; i<p.length/2; i++) {
		    reverse = p[i];
		    p[i] = p[p.length-1-i];
		    p[p.length-1-i] = reverse;
		}
		Graph g = new Graph("Stock Data Popularity (by Number of Transaction)");
		g.setVisible(true);
		g.plot(p);
		desktop.add(g);
		try {
		    g.setSelected(true);
		} catch (PropertyVetoException exc) {}	    	    
		
		StackDistance sdobj = new StackDistance();
		long[] sd = sdobj.calcStackDistance(sstr);
		saveArrayToFile(sd, "stockdata//savestackdistance");
		g = new Graph("Stock Data Stack Distance");
		g.setVisible(true);
		g.plot(sd);
		desktop.add(g);
		try {
		    g.setSelected(true);
		} catch (PropertyVetoException exc) {}
	    }
	    
	} // end if-else-if
    } // END ACTIONLISTENER ******
    
    public void itemStateChanged(ItemEvent e) {
    }
    
    //Every files should have the same y-dimension (width)
    private StockTuple[] multipleFilesToMatrix(File[] file, int width) {
	
	FileWriter fw = null;
	StockTuple[] output;
	int count = 0;
	ArrayList stockList = new ArrayList();
	
	File ini = new File("ini//stockNames.ini");
	
	if(ini.exists()) {
	    BufferedReader bfread = null;
	    try {
		bfread = new BufferedReader(new FileReader(ini));
	    } catch (FileNotFoundException e) {
		e.printStackTrace();
	    }
	    
	    while(true) {
		String sname = null;
		try {
		    sname = bfread.readLine();
		} catch (IOException e) {
		    e.printStackTrace();
		}
		if(sname == null) break;
		stockList.add(sname);
	    }
	    ini.delete();
	} else {
	    System.out.println("Warning: ini//stockNames.ini does not exists, enumerating stock from scratch.");
	}
	
	for(int i = 0; i<file.length; i++) {
	    BufferedReader in = null;
	    String s = null;				
	    try {
		fw = new FileWriter("stockdata//Analyze.txt", true);
	    } catch (IOException e) {
		e.printStackTrace();
	    }
	    
	    try {
		in = new BufferedReader(new FileReader(file[i]));				
		s = in.readLine();
	    } catch (IOException e) { e.printStackTrace(); }
	    
	    System.out.println(file[i].getName());
	    String name = enumerateStockName(s.substring(0, s.indexOf(' ')), stockList);
	    
	    while(s!=null) {
		
		StringTokenizer tok = new StringTokenizer(s);
		try {
		    fw.write(name + " ");
		    tok.nextToken();
		    tok.nextToken();
		    tok.nextToken();
		    tok.nextToken();					
		    s = tok.nextToken();
		    fw.write(String.valueOf(translateTime(s)));
		    fw.write("\r\n");
		    s = in.readLine();
		    
		} catch (IOException e) {
		    e.printStackTrace();
		}
		
		count++;
		tok = null;
	    }
	    
	    try {
		in.close();
		fw.flush();
	    } catch (IOException e) { e.printStackTrace(); }
	    
	    s = null;
	    in = null;
	    System.gc();
	}
	
	try {
	    fw.close();
	} catch (IOException e) { e.printStackTrace(); }
	
	output = new StockTuple[count];
	
	BufferedReader in = null;
	try {
	    in = new BufferedReader(new FileReader("stockdata//Analyze.txt"));
	} catch (FileNotFoundException e) {
	    e.printStackTrace();
	}
		
	for(int i = 0; i<count; i++) {
	    
	    String ss = null;
	    try {
		ss = in.readLine();
	    } catch (IOException e) {
		e.printStackTrace();
	    }
	    
	    output[i] = new StockTuple(
	        Integer.parseInt(ss.substring(0, ss.indexOf(" "))),
	        Integer.parseInt(ss.substring(ss.indexOf(" ") + 1, ss.length())));

	    ss = null;
	    if(i%1000 == 0) {
		System.gc();
		System.out.println("Completed " + i + " out of " + count);
	    }
	}
	
	try {
	    fw = new FileWriter("ini//stockNames.ini");
	    for(int i = 0; i<stockList.size(); i++) {
		fw.write((String)stockList.get(i) + "\r\n");
	    }
	    fw.close();		
	} catch (IOException e) {
	    e.printStackTrace();
	}
	
	return output;
    }
    
    private String enumerateStockName(String name, ArrayList stockList) {
	
	if(!stockList.contains(name))
	    stockList.add(name);
	
	return String.valueOf(stockList.indexOf(name));
	
    }
    
    private int translateTime(String s) {
	StringTokenizer tok = new StringTokenizer(s, ":.");
	
	String hour = tok.nextToken();
	if(hour.startsWith("0")) {
	    hour = hour.substring(1);
	}
	int HH = Integer.parseInt(hour);
	
	String minute = tok.nextToken();
	if(minute.startsWith("0")) {
	    minute = minute.substring(1);
	}
	int MM = Integer.parseInt(minute);
	
	String second = tok.nextToken();
	if(second.startsWith("0")) {
	    second = second.substring(1);
	}
	int SS = Integer.parseInt(second);
	
	String msecond = tok.nextToken();
	while(msecond.startsWith("0") && msecond.length() > 1) {
	    msecond = msecond.substring(1);
	}
	int mmm = Integer.parseInt(msecond);
	
	return HH*60*60*1000 + MM*60*1000 + SS*1000 + mmm;
    }
    
    private String[][] fileToMatrix(File file, int width) {
	String s = null;	
	BufferedReader in = null;
	StringTokenizer tok = null;
	ArrayList arr = new ArrayList();
	String[] temp;
	String[][] output;
	
	try {
	    in = new BufferedReader(new FileReader(file));
	} catch (FileNotFoundException e) { e.printStackTrace(); }
	
    	try {
	    s = in.readLine();
    	} catch (IOException e) { e.printStackTrace(); }
	
	while(s!=null) {
	    temp = new String[width];
	    
	    tok = new StringTokenizer(s,"\t\n\r\f, ");
	    
	    for(int j = 0; j<width; j++) {
		temp[j] = tok.nextToken();
	    }
	    
	    arr.add(temp);
	    
	    try {
		s = in.readLine();
	    } catch (IOException e) { e.printStackTrace(); }				
	}
	
	try {
	    in.close();
	} catch (IOException e) { e.printStackTrace(); }
	
	output = new String[arr.size()][width];
	for(int i = 0; i<arr.size(); i++) {
	    temp = (String[]) arr.get(i);
	    for(int j = 0; j<width; j++) {
		output[i][j] = temp[j]; 
	    }
	}
	
	return output;
    }
    
    
    private void saveArrayToFile(StockTuple[] stockTuple) {
	
	FileWriter fw = null;
	try {
	    fw = new FileWriter("stockdata//SortedAnalyze.txt", true);
	    for(int i = 0; i<stockTuple.length; i++) 
		fw.write(stockTuple[i].name + " " + 
			 String.valueOf(stockTuple[i].time) + "\r\n");
	    fw.flush();
	    fw.close();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	
    }
    
    private void saveArrayToFile(long[] a, String name) {
	
	FileWriter fw = null;
	try {
	    fw = new FileWriter(name + ".txt");
	    for(int i = 0; i<a.length; i++) 
		fw.write(String.valueOf(a[i]) + "\r\n");
	    fw.flush();
	    fw.close();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	
    }	
    
    class StockTuple {
	public StockTuple(int name, int time) {
	    this.name = name;
	    this.time = time;
	}
	
	public int name;
	public int time;
    }
    
    class StockTupleComparator implements Comparator {
	public int compare(Object o1, Object o2) {
	    return ((StockTuple)o1).time - ((StockTuple)o2).time;
	}
    }
    
    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() {
        ExStockDataInterface frame = new ExStockDataInterface();
        frame.setDefaultLookAndFeelDecorated(false);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }
    
    public static void main(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.

        javax.swing.SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    createAndShowGUI();
		}
	    });
    }    
}

