package ExStockData.Graph;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;

import java.util.ArrayList;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;

import javax.swing.JInternalFrame;
import javax.swing.JPanel;

import ExStockData.Graph.Popularity;
import ExStockData.Graph.StackDistance;

import ExStockDataInterface;

public class Graph extends JInternalFrame {
    private static final int xOffset = 30, yOffset = 30;
	private int xSize, ySize;
	private double xFactor, yFactor;
	private Graphics2D graphics;
	private StackDistance sd;
	public boolean dot = false;
	public long[] d;

    public Graph(String title, StackDistance sd) {
        super(title + " Graph #" + (++ExStockDataInterface.openFrameCount), 
              true, //resizable
              true, //closable
              false, //maximizable
              true);//iconifiable
    	xSize = ExStockDataInterface.XBOUND;
    	ySize = ExStockDataInterface.YBOUND;
    	xFactor = 1.0;
    	yFactor = 1.0;
        setSize(xSize,ySize);
        setBackground(Color.white);
        setLocation(xOffset*ExStockDataInterface.openFrameCount, 
		    yOffset*ExStockDataInterface.openFrameCount);
        this.sd = sd;
    }

    public Graph(String title) {
        super(title + " Graph #" + (++ExStockDataInterface.openFrameCount), 
              true, //resizable
              true, //closable
              false, //maximizable
              true);//iconifiable
    	xSize = ExStockDataInterface.XBOUND;
    	ySize = ExStockDataInterface.YBOUND;
    	xFactor = 1.0;
    	yFactor = 1.0;
        setSize(xSize,ySize);
        setBackground(Color.white);
        setLocation(xOffset*ExStockDataInterface.openFrameCount, 
		    yOffset*ExStockDataInterface.openFrameCount);
    }
    
    public void plot(int col1, int col2) {
	d = sd.calcStackDistance(col1, col2);
    }
    
    public void plot(long[] d) {
    	long max = d[0];
    	for(int i = 1; i<d.length; i++)
    		if(d[i]>max) max = d[i];
    	ySize = (int)(max/0.7);
    	xSize = (int)(d.length/0.8)*2;
    	
    	if(ySize < 200) {
	    yFactor = 200/ySize;
	    ySize = 200;
    	} else if(ySize > ExStockDataInterface.YBOUND) {
	    yFactor = ExStockDataInterface.YBOUND/(double)ySize;
	    ySize = ExStockDataInterface.YBOUND;   			
    	}
    	
    	if(xSize < 200) {
	    xFactor = 200/xSize;
	    xSize = 200;
    	} else if(xSize > ExStockDataInterface.XBOUND) {
	    xFactor = ExStockDataInterface.XBOUND/(double)xSize;
	    xSize = ExStockDataInterface.XBOUND;
    	}
        setSize(xSize,ySize);    	
    	this.d = d;
    }
    
    // a function plot a huge file    
    public void plot(File file) {    
    	ArrayList arr = new ArrayList();
	
	BufferedReader in = null;
	
       	try {
	    in = new BufferedReader(new FileReader(file));
	    do {
		String s = in.readLine();
		if(s == null) break;
		arr.add(s);
	    } while (true);
	    
    	} catch (Exception ee) {
	    ee.printStackTrace();
    	}
	
	d = new long[arr.size()];
	
    	for(int i = 0; i<arr.size(); i++) {
	    d[i] = Long.parseLong((String)arr.get(i));
    	}
    	
    	plot(d);
    }
    
    // a function plot popularity
    public void plotPopularity(File file) {    
	
	Popularity pop = new Popularity();
	d = pop.calcPopularityByObjNum(file);
	
	plot(d);
	
    }
    
    public void paint(Graphics g) {
	int min = Math.min(d.length,(int)(xSize*0.8*2));
	super.paint(g);
	graphics = (Graphics2D) g;
	
	graphics.setPaint(Color.blue);
	if(dot) {
	    for(int i=0; i<min; i++)
		graphics.draw(new Line2D.Double(xSize/10+i*2*xFactor,ySize*0.9-d[i]*yFactor-1,
						xSize/10+i*2*xFactor,ySize*0.9-d[i]*yFactor));
	} else {
	    for(int i=0; i<min; i++)
		graphics.draw(new Line2D.Double(xSize/10+i*2*xFactor,ySize*0.9,
						xSize/10+i*2*xFactor,ySize*0.9-d[i]*yFactor));
	}
	
	graphics.setPaint(Color.black);
	graphics.draw(new Line2D.Double(xSize/10,ySize*0.9,
					xSize/10,(ySize/10)+20));
	graphics.draw(new Line2D.Double(xSize/10,ySize*0.9,
					xSize*0.9,ySize*0.9));
    }   
}
