package ExStockData.Graph;

import java.io.File;

import java.util.StringTokenizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JInternalFrame;
import javax.swing.table.TableColumn;
import javax.swing.table.JTableHeader;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.event.ListSelectionEvent;

import ExStockDataInterface;

public class ListView extends JInternalFrame {
    private int xSize, ySize;
    private static final int xOffset = 15, yOffset = 15;
    private StackDistance sd;
    private TableModel tableModel;
    private int row, col;
    
    public ListView(String title, StackDistance sd) {
    	super(title + " List Data #" + (++ExStockDataInterface.openFrameCount),
	      true, //resizable
	      true, //closable
	      true, //maximizable
	      true);//iconifiable
    	this.sd = sd;
    	row = sd.rowCount;
    	col = sd.colCount;
	xSize = 300;
	ySize = 300;
	setSize(xSize, ySize);    	
	setBackground(Color.white);
	setLocation(xOffset*ExStockDataInterface.openFrameCount, 
		    yOffset*ExStockDataInterface.openFrameCount);
    	
    	tableModel = new TableModel(sd);
        JTable table = new JTable(tableModel);
        table.setPreferredScrollableViewportSize(new Dimension(500, 70));
        JScrollPane scrollPane = new JScrollPane(table);
        initColumnSizes(table);
        getContentPane().add(scrollPane);
    }
    
    public ListView(String title, String[] columnNames, Object[][] data) {
    	super(title + " (List Data #" + (++ExStockDataInterface.openFrameCount) + ")",
	      true, //resizable
	      true, //closable
	      true, //maximizable
	      true);//iconifiable
    	row = data.length;
    	col = data[0].length;    	
	
    	if(columnNames == null) {
	    columnNames = new String[col];
	    for(int i = 0; i<col; i++) {
		columnNames[i] = "data #" + i;
	    }
    	}
    	
	setBackground(Color.white);
	setLocation(xOffset*ExStockDataInterface.openFrameCount, 
		    yOffset*ExStockDataInterface.openFrameCount);
    	
    	tableModel = new TableModel(data, columnNames);
        MyTable table = new MyTable(this, tableModel);
	xSize = 140*col;
	
	if(row<40)
	    ySize = table.getRowHeight()*row+52;
	else ySize = table.getRowHeight()*40+52;
	
	setSize(xSize, ySize);
        table.setPreferredScrollableViewportSize(new Dimension(500, 70));
        JScrollPane scrollPane = new JScrollPane(table);
        initColumnSizes(table);
        getContentPane().add(scrollPane);
    }
    
    public void insert(int index, String[] obj) {
    	tableModel.insertRow(index, obj);
    }
    
    private void initColumnSizes(JTable table) {
        TableModel model = (TableModel)table.getModel();
        TableColumn column = null;
        Component comp = null;
        int headerWidth = 0;
        int cellWidth = 0;
        TableCellRenderer headerRenderer =
            table.getTableHeader().getDefaultRenderer();
	
        for (int i = 0; i < col; i++) {
            column = table.getColumnModel().getColumn(i);
	    
            comp = headerRenderer.getTableCellRendererComponent(
		   null, column.getHeaderValue(),
		   false, false, 0, 0);
            headerWidth = comp.getPreferredSize().width;
            cellWidth = comp.getPreferredSize().width;
            column.setPreferredWidth(Math.max(headerWidth, cellWidth));
        }
    }
    
    class MyTable extends JTable {
    	private JInternalFrame parent;
	private TableModel tableModel;    	
	
    	public MyTable(ListView parent, TableModel tableModel) {
	    super(tableModel);
	    this.parent = parent;
	    this.tableModel = tableModel;
    	}
	
	public void columnSelectionChanged(ListSelectionEvent e) {
	}
    }
    
    class TableModel extends DefaultTableModel {
        private int coloumCount;
        private String[] columnNames;
	private Object[][] data;
	private int row, col;
	
	public TableModel(StackDistance sd) {
	    super();
	    row = sd.rowCount;
	    col = sd.colCount;
	    StringTokenizer ttok = sd.tok;
	    columnNames = new String[col];
	    for(int i = 0; i<col; i++)
		columnNames[i] = "data" + i;
	    data = new Object[row][col];
	    for(int i = 0; i<row; i++) {
		for(int j = 0; j<col; j++)
		    data[i][j] = new Integer(Integer.parseInt(ttok.nextToken()));
	    }
	}
	
	public TableModel(Object[][] data, String[] columnNames) {
	    super(data, columnNames);
	    this.data = data;
	    this.columnNames = columnNames;
	    row = data.length;
	    col = data[0].length;
	}
    } //TableModel
}
