package ExStockData.Graph;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.StringTokenizer;


public class StackDistance {
    public int rowCount;
    public int colCount;
    public boolean isSelf;
    public long[] d1;
    public long[] d2;
    public long[] stackDistance;
    public StringTokenizer tok;
    
    public StackDistance() {}
    
    public StackDistance(File file) {
	rowCount = -1;
	colCount = 0;
	
	BufferedReader in = null;
	String s = "";
	String str = "";
	
	try {
	    in = new BufferedReader(new FileReader(file));
	} catch (FileNotFoundException e) { e.printStackTrace(); }
	
	while(s!=null) {
	    str+=s;
	    rowCount++;
	    try {
		s = in.readLine();
	    } catch (IOException e) { e.printStackTrace(); }
	}
	
	tok = new StringTokenizer(str,"\t\n\r\f, ");
	colCount = tok.countTokens()/rowCount;
    }
    
    public StackDistance(File file1, File file2) {
	rowCount = -1;
	colCount = 0;
	
	BufferedReader in1 = null;
	BufferedReader in2 = null;
	String s1 = "";
	String s2 = "";
	String str = "";
	
	try {
	    in1 = new BufferedReader(new FileReader(file1));
	    in2 = new BufferedReader(new FileReader(file2));
	} catch (FileNotFoundException e) { e.printStackTrace(); }
	
	while(s1!=null || s2!=null) {
	    if(s1!=null)
		str+=s1;
	    else str+="-1,-1,-1,";
	    if(!str.endsWith(","))
		str+=",";
	    
	    if(s2!=null)
		str+=s2;
		    else str+="-1,-1,-1,";
	    if(!str.endsWith(","))
		str+=",";
	    
	    rowCount++;
	    try {
		s1 = in1.readLine();
		s2 = in2.readLine();				
	    } catch (IOException e) { e.printStackTrace(); }
	}
	
	tok = new StringTokenizer(str,"\t\n\r\f, ");
	colCount = tok.countTokens()/rowCount;
    }
    
    public long[] calcStackDistance(int col1, int col2) {
	StringTokenizer ttok = tok;
	ArrayList ds = new ArrayList();
	isSelf = false;
	if(col1 == col2) isSelf = true;
	
	d1 = new long[rowCount];
	d2 = new long[rowCount];
	
	for(int i = 0; i<rowCount; i++) {
	    for(int j = 0; j<col1; j++)
		ttok.nextToken();
	    d1[i]=Integer.parseInt(ttok.nextToken());
	    if(isSelf) {
		d2[i] = d1[i];
	    } else {
		for(int j = col1+1; j<col2; j++)
		    ttok.nextToken();
		d2[i]=Integer.parseInt(ttok.nextToken());
	    }
	    for(int j = col2+1; j<colCount; j++)
		ttok.nextToken();
	}
	
	for(int i = 0; i<rowCount; i++) {
	    int j = i;
	    
	    for(;j>0; j--) {
		if(isSelf && i==j)
		    continue;
		if(d1[i]==d2[j]) {
		    ds.add(new Long(i-j));
		    break;
		}
	    }
	    
	    if(j==-1)
		ds.add(new Integer(-1));
	}
	
	Long temp;
	long[] ids = new long[ds.size()];
	for(int i=0; i<ds.size(); i++) {			
	    temp = (Long)(ds.get(i));
	    ids[i] = temp.longValue();
	}
		
	Arrays.sort(ids);
	long[] output = new long[(int)(ids[ids.length-1])+1];
	for(int i=0; i<output.length; i++)
			output[i]=0;
	for(int i=0; i<ids.length; i++)
	    output[(int)ids[i]]++;
	
	return output;
    }
    
    
    public long[] calcStackDistance(String[][] obj, int index) {
    	String[] output = new String[obj.length];
    	for(int i = 0; i<obj.length; i++) {
	    output[i] = obj[i][index];
	}
	return calcStackDistance(output);
    }
    
    public long[] calcStackDistance(String[] obj) {
	ArrayList sd = new ArrayList();
	ArrayList shown;
	String cur;
	for(int i = 0; i<obj.length; i++) {
	    shown = new ArrayList();
	    cur = obj[i];
	    for(int j = i-1; j>=0; j--) {
		if(cur.compareTo((String)obj[j]) == 0) {
		    sd.add(new Long(shown.size()));
		    break;
		} else {
		    if(!shown.contains(obj[j]))
			shown.add(obj[j]);
		}
	    }
	}
	
	long[] ids = new long[sd.size()];
	stackDistance = new long[sd.size()];
	for(int i=0; i<sd.size(); i++) {
	    ids[i] = ((Long)sd.get(i)).longValue();
	    stackDistance[i] = ids[i];
	}
	
	Arrays.sort(ids);
	long[] output = new long[(int)(ids[ids.length-1])+1];
	for(int i=0; i<output.length; i++)
	    output[i]=0;
	for(int i=0; i<ids.length; i++)
	    output[(int)ids[i]]++;
	
	return output;
    }    
    
    public long[] calcNonUniqueStackDistance(String[][] obj, int index) {
    	String[] output = new String[obj.length];
    	for(int i = 0; i<obj.length; i++) {
	    output[i] = obj[i][index];
	}
	return calcNonUniqueStackDistance(output);
    }
    
    public long[] calcNonUniqueStackDistance(String[] obj) {
	long[] output = new long[obj.length];
	int count = 0;
	String cur;
	
	for(int i = 0; i<obj.length; i++) {
	    output[i] = 0;
	    count = 0;
	    cur = obj[i];
	    int j = 0;
	    for(j = i-1; j>=0; j--) {
		if(cur.compareTo((String)obj[j]) == 0) {
		    output[i] = count;
		    break;
		} else {
		    count++;
		}
	    }
	    if(j<0)
		output[i] = -1;
	}

	return output;
    }    
    
    
    public long[] calcCombineNonUniqueStackDistance(String[][] obj1, 
						    String[][] obj2, 
						    int objIndex, 
						    int timeIndex) {
    	StackDistanceTuple[] sdt = new StackDistanceTuple[obj1.length + obj2.length];
    	
    	for(int i = 0; i<sdt.length; i++)
	    sdt[i] = new StackDistanceTuple();
	
    	for(int i = 0; i<obj1.length; i++) {    		
	    sdt[i].obj = obj1[i][objIndex];
	    sdt[i].time = obj1[i][timeIndex];
	    sdt[i].which = true;
	}
	
	for(int i = obj1.length; i<obj1.length+obj2.length; i++) {
	    sdt[i].obj = obj2[i-obj1.length][objIndex];
	    sdt[i].time = obj2[i-obj1.length][timeIndex];
	    sdt[i].which = false;
	}
	
	Arrays.sort(sdt, new StackDistanceComparator());   	
	return calcCombineNonUniqueStackDistance(sdt);
    }
    
    public long[] calcCombineNonUniqueStackDistance(StackDistanceTuple[] object) {
	long[] output = new long[object.length];
	int count = 0;
	StackDistanceTuple cur;
	
	for(int i = 0; i<object.length; i++) {
	    output[i] = 0;
	    count = 0;
	    cur = object[i];
	    int j = 0;
	    for(j = i-1; j>=0; j--) {
		if(((StackDistanceTuple)object[j]).which == cur.which && 
		   cur.obj.compareTo(((StackDistanceTuple)object[j]).obj) == 0) {
		    output[i] = count;
		    break;
		} else {
		    count++;
		}
	    }
	    if(j<0)
		output[i] = -1;
	}
	
	return output;
    }    
    
    
    class StackDistanceTuple {
	
    	public StackDistanceTuple() {
	    obj = "";
	    time = "";
	    which = false;
    	}
    	
    	public String obj;
    	public String time;
    	public boolean which;
    	
    }
    
    class StackDistanceComparator implements Comparator {
	public int compare(Object o1, Object o2) {
	    return Integer.parseInt(((StackDistanceTuple)o1).time) -
		Integer.parseInt(((StackDistanceTuple)o2).time);
	}
    }
    
}
